@@ -180,12 +180,13 @@ sub retry_job {
 
   return !!$self->pg->db->query(
     "update minion_jobs
-     set delayed = (now() + (interval '1 second' * ?)),
+     set attempts = coalesce(?, attempts),
+       delayed = (now() + (interval '1 second' * ?)),
        priority = coalesce(?, priority), queue = coalesce(?, queue),
        retried = now(), retries = retries + 1, state = 'inactive'
      where id = ? and retries = ?
-     returning 1", $options->{delay} // 0, @$options{qw(priority queue)}, $id,
-    $retries
+     returning 1", $options->{attempts}, $options->{delay} // 0,
+    @$options{qw(priority queue)}, $id, $retries
   )->rows;
 }
 
@@ -242,16 +243,17 @@ sub _try {
      set started = now(), state = 'active', worker = ?
      where id = (
        select id from minion_jobs as j
-       where delayed <= now() and (parents = '{}' or not exists (
-         select 1 from minion_jobs
-         where id = any (j.parents)
-           and state in ('inactive', 'active', 'failed')
-       )) and queue = any (?) and state = 'inactive' and task = any (?)
+       where delayed <= now() and id = coalesce(?, id)
+         and (parents = '{}' or not exists (
+           select 1 from minion_jobs
+           where id = any (j.parents)
+             and state in ('inactive', 'active', 'failed')
+         )) and queue = any (?) and state = 'inactive' and task = any (?)
        order by priority desc, id
        limit 1
        for update skip locked
      )
-     returning id, args, retries, task", $id,
+     returning id, args, retries, task", $id, $options->{id},
     $options->{queues} || ['default'], [keys %{$self->minion->tasks}]
   )->expand->hash;
 }
@@ -331,6 +333,12 @@ These options are currently available:
 
 =over 2
 
+=item id
+
+  id => '10023'
+
+Dequeue a specific job.
+
 =item queues
 
   queues => ['important']
@@ -689,6 +697,12 @@ These options are currently available:
 
 =over 2
 
+=item attempts
+
+  attempts => 25
+
+Number of times performing this job will be attempted.
+
 =item delay
 
   delay => 10
@@ -943,12 +957,23 @@ create table if not exists minion_locks (
   name    text not null,
   expires timestamp with time zone not null
 );
-create function minion_lock(text, int, int) returns bool as $$
+
+-- 16 down
+drop table if exists minion_locks;
+
+-- 17 up
+alter table minion_jobs add column notes jsonb
+  check(jsonb_typeof(notes) = 'object') not null default '{}';
+alter table minion_locks set unlogged;
+create index on minion_locks (name, expires);
+
+-- 18 up
+create or replace function minion_lock(text, int, int) returns bool as $$
 declare
   new_expires timestamp with time zone = now() + (interval '1 second' * $2);
 begin
-  delete from minion_locks where expires < now();
   lock table minion_locks in exclusive mode;
+  delete from minion_locks where expires < now();
   if (select count(*) >= $3 from minion_locks where name = $1) then
     return false;
   end if;
@@ -959,12 +984,5 @@ begin
 end;
 $$ language plpgsql;
 
--- 16 down
+-- 18 down
 drop function if exists minion_lock(text, int, int);
-drop table if exists minion_locks;
-
--- 17 up
-alter table minion_jobs add column notes jsonb
-  check(jsonb_typeof(notes) = 'object') not null default '{}';
-alter table minion_locks set unlogged;
-create index on minion_locks (name, expires);
